# -*- coding:utf-8 -*-
import time
import sys
import argparse
import kdxfsdk.config_defaults


class PCA9685:
    """
    PWM motor controler using PCA9685 boards.
    This is used for most RC Cars
    """
    def __init__(self, channel, frequency=60):
        import Adafruit_PCA9685
        # Initialise the PCA9685 using the default address (0x40).
        self.pwm = Adafruit_PCA9685.PCA9685()
        self.pwm.set_pwm_freq(frequency)
        self.channel = channel

    def set_pulse(self, pulse):
        try:
            self.pwm.set_pwm(self.channel, 0, pulse)
        except OSError as err:
            print("Unexpected issue setting PWM (check wires to motor board): {0}".format(err))


class FourFootRobot(object):
    def __init__(self):
        direction0 = PCA9685(0)
        direction1 = PCA9685(2)
        direction2 = PCA9685(4)
        direction3 = PCA9685(6)
        self.DirectionPwm = [direction0, direction1, direction2, direction3]

        foot0 = PCA9685(1)
        foot1 = PCA9685(3)
        foot2 = PCA9685(5)
        foot3 = PCA9685(7)
        self.Foot = [foot0, foot1, foot2, foot3]

    def in_line(self):
        # direction
        for i in range(4):
            self.DirectionPwm[i].set_pulse(config_defaults.DIRECTION_CONFIG[i][1])

    def stable(self):
        # direction
        for i in range(4):
            self.DirectionPwm[i].set_pulse(config_defaults.DIRECTION_CONFIG[i][2])

    def stand_up(self):
        self.in_line()
        # foot
        for i in range(4):
            self.Foot[i].set_pulse(config_defaults.FOOT_CONFIGFOOT_CONFIG[i][1])

    def set_one(self, i, pulse):
       self.Foot[i].set_pulse(pulse)

    def sit(self):
        # direction
        self.in_line()
        # foot
        for i in range(4):
            self.Foot[i].set_pulse(config_defaults.FOOT_CONFIG[i][0])

    # 1, means forward, -1 means backword
    def _right_move_begin(self, for_or_back):
        if for_or_back == 1:
            up_delta = (config_defaults.FOOT_CONFIG[1][1] + config_defaults.FOOT_CONFIG[1][2])/2
        else:
            up_delta = (config_defaults.FOOT_CONFIG[1][1] + config_defaults.FOOT_CONFIG[1][0])/2
        self.Foot[1].set_pulse(int(up_delta))
        time.sleep(config_defaults.MOVE_INTERVAL * config_defaults.MOVE_INTERVAL_RATIO)
        if for_or_back == 1:
            up_delta = (config_defaults.FOOT_CONFIG[2][1] + config_defaults.FOOT_CONFIG[2][0])/2
        else:
            up_delta = (config_defaults.FOOT_CONFIG[2][1] + config_defaults.FOOT_CONFIG[2][2])/2

        self.Foot[2].set_pulse(int(up_delta))
        time.sleep(config_defaults.MOVE_INTERVAL * config_defaults.MOVE_INTERVAL_RATIO)
        self.Foot[0].set_pulse(config_defaults.FOOT_CONFIG[0][0])
        self.Foot[3].set_pulse(config_defaults.FOOT_CONFIG[3][0])
        time.sleep(config_defaults.MOVE_INTERVAL * config_defaults.MOVE_INTERVAL_RATIO * 1.5)

    def _right_move_end(self, for_or_back):
        if for_or_back == 1:
            up_delta1 = (config_defaults.FOOT_CONFIG[1][1] + config_defaults.FOOT_CONFIG[1][0])/2
        else:
            up_delta1 = (config_defaults.FOOT_CONFIG[1][1] + config_defaults.FOOT_CONFIG[1][2])/2
        self.Foot[1].set_pulse(int(up_delta1))
        time.sleep(config_defaults.MOVE_INTERVAL * config_defaults.MOVE_INTERVAL_RATIO)
        if for_or_back == 1:
            up_delta2 = (config_defaults.FOOT_CONFIG[2][1] + config_defaults.FOOT_CONFIG[2][2])/2
        else:
            up_delta2 = (config_defaults.FOOT_CONFIG[2][1] + config_defaults.FOOT_CONFIG[2][0])/2
        self.Foot[2].set_pulse(int(up_delta2))
        time.sleep(config_defaults.MOVE_INTERVAL * config_defaults.MOVE_INTERVAL_RATIO * 1.5)

    def _left_move_begin(self, for_or_back):
        if for_or_back == 1:
            up_delta = (config_defaults.FOOT_CONFIG[0][1] + config_defaults.FOOT_CONFIG[0][2])/2
        else:
            up_delta = (config_defaults.FOOT_CONFIG[0][1] + config_defaults.FOOT_CONFIG[0][0])/2
        self.Foot[0].set_pulse(int(up_delta))
        time.sleep(config_defaults.MOVE_INTERVAL * config_defaults.MOVE_INTERVAL_RATIO)
        if for_or_back == 1:
            up_delta = (config_defaults.FOOT_CONFIG[3][1] + config_defaults.FOOT_CONFIG[3][0])/2
        else:
            up_delta = (config_defaults.FOOT_CONFIG[3][1] + config_defaults.FOOT_CONFIG[3][2])/2

        self.Foot[3].set_pulse(int(up_delta))
        self.Foot[2].set_pulse(config_defaults.FOOT_CONFIG[2][0])
        self.Foot[1].set_pulse(config_defaults.FOOT_CONFIG[1][0])
        time.sleep(config_defaults.MOVE_INTERVAL * config_defaults.MOVE_INTERVAL_RATIO * 1.5)

    def _left_move_end(self, for_or_back):
        if for_or_back == 1:
            up_delta1 = (config_defaults.FOOT_CONFIG[0][1] + config_defaults.FOOT_CONFIG[0][0])/2
        else:
            up_delta1 = (config_defaults.FOOT_CONFIG[0][1] + config_defaults.FOOT_CONFIG[0][2])/2
        self.Foot[0].set_pulse(int(up_delta1))
        time.sleep(config_defaults.MOVE_INTERVAL * config_defaults.MOVE_INTERVAL_RATIO)
        if for_or_back == 1:
            up_delta2 = (config_defaults.FOOT_CONFIG[3][1] + config_defaults.FOOT_CONFIG[3][2])/2
        else:
            up_delta2 = (config_defaults.FOOT_CONFIG[3][1] + config_defaults.FOOT_CONFIG[3][0]) / 2
        self.Foot[3].set_pulse(int(up_delta2))
        time.sleep(config_defaults.MOVE_INTERVAL * config_defaults.MOVE_INTERVAL_RATIO * 1.5)

    def tern_left(self):
        # direction
        for i in range(4):
            self.Foot[i].set_pulse(config_defaults.FOOT_CONFIG[i][0])
            self.DirectionPwm[i].set_pulse(config_defaults.DIRECTION_CONFIG[i][3])
            time.sleep(config_defaults.MOVE_INTERVAL * config_defaults.MOVE_INTERVAL_RATIO)
        self.stable()

    def forword(self):
        self._right_move_begin(1)
        time.sleep(config_defaults.MOVE_INTERVAL * config_defaults.MOVE_INTERVAL_RATIO)
        self._right_move_end(1)
        self._left_move_begin(1)
        time.sleep(config_defaults.MOVE_INTERVAL * config_defaults.MOVE_INTERVAL_RATIO)
        self._left_move_end(1)
        time.sleep(config_defaults.MOVE_INTERVAL * config_defaults.MOVE_INTERVAL_RATIO)

    def backword(self):
        self._right_move_begin(-1)
        time.sleep(config_defaults.MOVE_INTERVAL * config_defaults.MOVE_INTERVAL_RATIO)
        self._right_move_end(-1)
        self._left_move_begin(-1)
        time.sleep(config_defaults.MOVE_INTERVAL * config_defaults.MOVE_INTERVAL_RATIO)
        self._left_move_end(-1)
        time.sleep(config_defaults.MOVE_INTERVAL * config_defaults.MOVE_INTERVAL_RATIO)

    def reset_inline(self):
        self.in_line()
        self.sit()

    def reset_stable(self):
        self.stable()
        self.stand_up()



def parse_args(args):
    parser = argparse.ArgumentParser(prog='calibrate', usage='%(prog)s [options]')
    parser.add_argument('--channel', help='The channel youd like to calibrate [0-15]')
    parsed_args = parser.parse_args(args)
    return parsed_args


def calibrate():
    args = sys.argv[1:]
    args = parse_args(args)
    channel = int(args.channel)
    c = PCA9685(channel)
    while True:
        try:
            val = input("""Enter a PWM setting to test.py ('q' for quit) (0-1500): """)
            if val == 'q' or val == 'Q':
                break
            pmw = int(val)
            c.set_pulse(pmw)
        except KeyboardInterrupt:
            print("\nKeyboardInterrupt received, exit.")
            break
        except Exception as ex:
            print("Oops, {}".format(ex))


if __name__ != '__main__':
    calibrate()

if __name__ == '__main__':
    foot_robot = FourFootRobot()
    foot_robot.sit()
    time.sleep(2)
    foot_robot.stand_up()
    for i in range(10):
        foot_robot.forword()
        time.sleep(0.1)
    foot_robot.sit()



import logging
import os
from robot import constants
from logging.handlers import RotatingFileHandler

PAGE = 4096

DEBUG = logging.DEBUG
INFO = logging.INFO
WARNING = logging.WARNING
ERROR = logging.ERROR


def tail(filepath, n=10):
    """
    实现 tail -n
    """
    res = ""
    with open(filepath, 'rb') as f:
        f_len = f.seek(0, 2)
        rem = f_len % PAGE
        page_n = f_len // PAGE
        r_len = rem if rem else PAGE
        while True:
            # 如果读取的页大小>=文件大小，直接读取数据输出
            if r_len >= f_len:
                f.seek(0)
                lines = f.readlines()[::-1]
                break

            f.seek(-r_len, 2)
            # print('f_len: {}, rem: {}, page_n: {}, r_len: {}'.format(f_len, rem, page_n, r_len))
            lines = f.readlines()[::-1]
            count = len(lines) -1   # 末行可能不完整，减一行，加大读取量

            if count >= n:  # 如果读取到的行数>=指定行数，则退出循环读取数据
                break
            else:   # 如果读取行数不够，载入更多的页大小读取数据
                r_len += PAGE
                page_n -= 1

    for line in lines[:n][::-1]:
        res += line.decode('utf-8')
    return res

def getLogger(name):
    """ 
    作用同标准模块 logging.getLogger(name) 
    
    :returns: logger
    """
    formatter = logging.Formatter('%(asctime)s - %(name)s - %(filename)s - %(funcName)s - line %(lineno)s - %(levelname)s - %(message)s')

    logger = logging.getLogger(name)
    logger.setLevel(logging.INFO)
    
    # FileHandler
    file_handler = RotatingFileHandler(os.path.join(constants.TEMP_PATH, 'xiaoli.log'), maxBytes=1024*1024,backupCount=5)
    file_handler.setLevel(level=logging.DEBUG)
    file_handler.setFormatter(formatter)
    logger.addHandler(file_handler)

    return logger

def readLog(lines=200):
    """ 
    获取最新的指定行数的 log

    :param lines: 最大的行数
    :returns: 最新指定行数的 log
    """
    log_path = os.path.join(constants.TEMP_PATH, 'xiaoli.log')
    if os.path.exists(log_path):
        return tail(log_path, lines)
    return ''
    



# -*- coding:utf-8 -*-
#!/usr/bin/env python

## recordtest.py
##
## This is an example of a simple sound capture script.
##
## The script opens an ALSA pcm device for sound capture, sets
## various attributes of the capture, and reads in a loop,
## writing the data to standard out.
##
## To test it out do the following:
## python recordtest.py out.raw # talk to the microphone
## aplay -r 8000 -f S16_LE -c 1 out.raw

#!/usr/bin/env python

from __future__ import print_function

import sys
import time
import getopt
import wave
import alsaaudio
import numpy as np

def usage():
    print('usage: recordtest.py [-d <device>] <file>', file=sys.stderr)
    sys.exit(2)


if __name__ == '__main__':

    device = 'default'

    opts, args = getopt.getopt(sys.argv[1:], 'd:')
    for o, a in opts:
        if o == '-d':
            device = a

    if not args:
        usage()

    f = wave.open(args[0], 'wb')

    # Open the device in nonblocking capture mode. The last argument could
    # just as well have been zero for blocking mode. Then we could have
    # left out the sleep call in the bottom of the loop
    inp = alsaaudio.PCM(alsaaudio.PCM_CAPTURE, alsaaudio.PCM_NONBLOCK, device=device)

    # Set attributes: Mono, 44100 Hz, 16 bit little endian samples
    inp.setchannels(1)
    inp.setrate(16000)
    inp.setformat(alsaaudio.PCM_FORMAT_S16_LE)

    f.setnchannels(1)
    f.setsampwidth(2)    #PCM_FORMAT_S16_LE
    f.setframerate(16000)


    print('%d channels, %d sampling rate\n' % (f.getnchannels(),
                                               f.getframerate()))
    # The period size controls the internal number of frames per period.
    # The significance of this parameter is documented in the ALSA api.
    # For our purposes, it is suficcient to know that reads from the device
    # will return this many frames. Each frame being 2 bytes long.
    # This means that the reads below will return either 320 bytes of data
    # or 0 bytes of data. The latter is possible because we are in nonblocking
    # mode.
    inp.setperiodsize(160)

    # loops = 1000000
    # while loops > 0:
    #     loops -= 1
    #     # Read data from device
    #     l, data = inp.read()
    #
    #     if l:
    #         f.writeframes(data)
    #         time.sleep(.001)
    recording = False
    frames = []
    THRESHOLD = 1000
    # 遗弃前12帧
    for i in range(0, 5):
        l, data = inp.read()
        time.sleep(.051)

    while (True):
        if not recording:
            print('检测中... ')
            # 采集小段声音
            frames = []
            for i in range(0, 4):
                l, data = inp.read()
                if l > 0:
                    time.sleep(.01)
                    print("some sound ")
                    frames.append(data)
                    a = np.fromstring(data, dtype='int16')
                    print(np.abs(a).mean())
            audio_data = np.frombuffer(b''.join(frames), dtype=np.int16)
            large_sample_count = np.sum(audio_data >= THRESHOLD * 0.6)
            time.sleep(.051)
            # 如果有符合条件的声音，则开始录制
            if large_sample_count >= THRESHOLD * 0.8:
                print("检测到信号")
                recording = True
            print(large_sample_count)
        else:
            nowavenum = 0
            while True:
                print("持续录音中...")
                subframes = []
                for i in range(0, 5):
                    l, data = inp.read()
                    if l > 0:
                        time.sleep(.001)
                        subframes.append(data)
                        frames.append(data)
                        a = np.fromstring(data, dtype='int16')
                        print(np.abs(a).mean())
                audio_data = np.frombuffer(b''.join(subframes), dtype=np.int16)
                if audio_data.size != 0:
                    temp = np.max(audio_data)
                    if temp <= THRESHOLD * 0.8:
                        nowavenum += 1
                    else:
                        nowavenum = 0

                    if nowavenum >= 2:
                        print("等待超时，开始保存")
                        frames.pop()
                        f.writeframes(b''.join(frames))
                        f.close()
                        break
            break



# -*- coding:utf-8 -*-
from __future__ import print_function

import os
from snowboy import snowboydecoder
from robot import utils, Player, constants, logging

logger = logging.getLogger(__name__)


class Audio(object):
    def __init__(self):
        self.interrupted = False
        self.record_file = ""

    def detected_callback(self):
        def start_record():
            utils.setRecordable(True)
            logger.info('开始录音')

        # first disable audio record.
        utils.setRecordable(False)
        # no after player played the startup beep.
        Player.play(constants.getData('beep_hi.wav'), onCompleted=start_record, wait=True)

    def audioRecorderCallback(self, fname):
        self.interrupted = True
        self.record_file = fname

    def _interrupt_callback(self):
        return self.interrupted

    def start_record(self):
        detector = snowboydecoder.HotwordDetector("/tmp/snowboy/resources/zijinshan.pmdl", sensitivity=0.5, audio_gain=1)
        detector.start(detected_callback=self.detected_callback,
                       audio_recorder_callback=self.audioRecorderCallback,
                       interrupt_check=self._interrupt_callback,
                       )
        return self.record_file

    # 这个其实是wav，为了跟之前的接口保持一致。
    def play(self, file_name):
        Player.play(file_name, wait=True)

    def wav_to_pcm(self, wav_file):
        pcm_file = "%s.pcm" % (wav_file.split(".")[0])
        os.system("ffmpeg -y  -i %s  -acodec pcm_s16le -f s16le -ac 1 -ar 16000 %s" % (
            wav_file, pcm_file))
        return pcm_file
    #
    # def play_pcm(self, pcm_file):
    #     # os.close(sys.stderr.fileno())
    #     with open(pcm_file, 'rb') as pcmfile:
    #         pcmdata = pcmfile.read()
    #     with wave.open('speach.wav', 'wb') as wavfile:
    #         wavfile.setparams((1, 2, 16000, 0, 'NONE', 'NONE'))
    #         wavfile.writeframes(pcmdata)
    #     self.play_wav('speach.wav')


if __name__ == "__main__":
    audio = Audio()
    filename = audio.start_record()
    print(filename)



# -*- coding: utf-8-*-
import subprocess
import os
import platform
from . import utils, logging
import _thread as thread
from ctypes import CFUNCTYPE, c_char_p, c_int, cdll
from contextlib import contextmanager

logger = logging.getLogger(__name__)

def py_error_handler(filename, line, function, err, fmt):
    pass

ERROR_HANDLER_FUNC = CFUNCTYPE(None, c_char_p, c_int, c_char_p, c_int, c_char_p)

c_error_handler = ERROR_HANDLER_FUNC(py_error_handler)

@contextmanager
def no_alsa_error():
    try:
        asound = cdll.LoadLibrary('libasound.so')
        asound.snd_lib_error_set_handler(c_error_handler)
        yield
        asound.snd_lib_error_set_handler(None)
    except:
        yield
        pass

def play(fname, onCompleted=None, wait=False):
    player = getPlayerByFileName(fname)
    player.play(fname, onCompleted=onCompleted, wait=wait)

def getPlayerByFileName(fname):
    foo, ext = os.path.splitext(fname)
    if ext in ['.mp3', '.wav']:
        return SoxPlayer()

class AbstractPlayer(object):

    def __init__(self, **kwargs):
        super(AbstractPlayer, self).__init__()

    def play(self):
        pass

    def play_block(self):
        pass

    def stop(self):
        pass

    def is_playing(self):
        return False
    

class SoxPlayer(AbstractPlayer):
    SLUG = 'SoxPlayer'

    def __init__(self, **kwargs):
        super(SoxPlayer, self).__init__(**kwargs)
        self.playing = False
        self.proc = None
        self.delete = False
        self.onCompleteds = []

    def doPlay(self):
        cmd = ['play', str(self.src)]
        logger.debug('Executing %s', ' '.join(cmd))
        self.proc = subprocess.Popen(cmd, stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL)
        self.playing = True
        self.proc.wait()
        self.playing = False
        if self.delete:
            utils.check_and_delete(self.src)
        logger.debug('play completed')
        if self.proc.returncode == 0:
            for onCompleted in self.onCompleteds:
                if onCompleted is not None:
                    onCompleted()
    
    def play(self, src, delete=False, onCompleted=None, wait=False):
        if os.path.exists(src) or src.startswith('http'):
            self.src = src
            self.delete = delete
            if onCompleted is not None:
                self.onCompleteds.append(onCompleted)
            if not wait:
                thread.start_new_thread(self.doPlay, ())
            else:
                self.doPlay()
        else:
            logger.critical('path not exists: {}'.format(src))

    def appendOnCompleted(self, onCompleted):
        if onCompleted is not None:
            self.onCompleteds.append(onCompleted)

    def play_block(self):
        self.run()

    def stop(self):
        if self.proc:
            self.onCompleteds = []
            self.proc.terminate()
            if self.delete:
                utils.check_and_delete(self.src)

    def is_playing(self):
        return self.playing


class MusicPlayer(SoxPlayer):
    """
    给音乐播放器插件使用的，
    在 SOXPlayer 的基础上增加了列表的支持，
    并支持暂停和恢复播放
    """
    SLUG = 'MusicPlayer'

    def __init__(self, playlist, plugin, **kwargs):
        super(MusicPlayer, self).__init__(**kwargs)
        self.playlist = playlist
        self.plugin = plugin
        self.idx = 0
        self.pausing = False
        self.last_paused = None

    def update_playlist(self, playlist):
        super().stop()
        self.playlist = playlist
        self.idx = 0
        self.play()

    def play(self):
        logger.debug('MusicPlayer play')
        path = self.playlist[self.idx]
        super().stop()
        super().play(path, False, self.next)

    def next(self):
        logger.debug('MusicPlayer next')
        super().stop()
        self.idx = (self.idx+1) % len(self.playlist)
        self.play()

    def prev(self):
        logger.debug('MusicPlayer prev')
        super().stop()
        self.idx = (self.idx-1) % len(self.playlist)
        self.play()

    def pause(self):
        logger.debug('MusicPlayer pause')
        self.pausing = True

    def stop(self):
        if self.proc:
            logger.debug('MusicPlayer stop')
            # STOP current play process
            self.last_paused = utils.write_temp_file(str(self.proc.pid), 'pid', 'w')
            self.onCompleteds = []
            subprocess.run(['pkill', '-STOP', '-F', self.last_paused])

    def resume(self):
        logger.debug('MusicPlayer resume')
        self.pausing = False
        self.onCompleteds = [self.next]
        if self.last_paused is not None:
            print(self.last_paused)
            subprocess.run(['pkill', '-CONT', '-F', self.last_paused])

    def is_playing(self):
        return self.playing

    def is_pausing(self):
        return self.pausing

    def turnUp(self):
        system = platform.system()
        if system == 'Darwin':
            res = subprocess.run(['osascript', '-e', 'output volume of (get volume settings)'], shell=False, capture_output=True, universal_newlines=True)
            volume = int(res.stdout.strip())
            volume += 20
            if volume >= 100:
                volume = 100
                self.plugin.say('音量已经最大啦', wait=True)
            subprocess.run(['osascript', '-e', 'set volume output volume {}'.format(volume)])
        elif system == 'Linux':
            res = subprocess.run(["amixer sget Master | grep 'Mono:' | awk -F'[][]' '{ print $2 }'"], shell=True, capture_output=True, universal_newlines=True)
            print(res.stdout)
            if res.stdout != '' and res.stdout.strip().endswith('%'):
                volume = int(res.stdout.strip().replace('%', ''))
                volume += 20
                if volume >= 100:
                    volume = 100
                    self.plugin.say('音量已经最大啦', wait=True)
                subprocess.run(['amixer', 'set', 'Master', '{}%'.format(volume)])
            else:
                subprocess.run(['amixer', 'set', 'Master', '20%+'])
        else:
            self.plugin.say('当前系统不支持调节音量', wait=True)
        self.resume()

    def turnDown(self):
        system = platform.system()
        if system == 'Darwin':
            res = subprocess.run(['osascript', '-e', 'output volume of (get volume settings)'], shell=False, capture_output=True, universal_newlines=True)
            volume = int(res.stdout.strip())
            volume -= 20
            if volume <= 20:
                volume = 20
                self.plugin.say('音量已经很小啦', wait=True)
            subprocess.run(['osascript', '-e', 'set volume output volume {}'.format(volume)])
        elif system == 'Linux':
            res = subprocess.run(["amixer sget Master | grep 'Mono:' | awk -F'[][]' '{ print $2 }'"], shell=True, capture_output=True, universal_newlines=True)
            if res.stdout != '' and res.stdout.endswith('%'):
                volume = int(res.stdout.replace('%', '').strip())
                volume -= 20
                if volume <= 20:
                    volume = 20
                    self.plugin.say('音量已经最小啦', wait=True)
                subprocess.run(['amixer', 'set', 'Master', '{}%'.format(volume)])
            else:
                subprocess.run(['amixer', 'set', 'Master', '20%-'])
        else:
            self.plugin.say('当前系统不支持调节音量', wait=True)
        self.resume()



# -*- coding: utf-8-*-
import yaml
import logging
import os
from . import constants

logger = logging.getLogger(__name__)

_config = {}
has_init = False

def reload():
    """
    重新加载配置
    """
    logger.info('配置文件发生变更，重新加载配置文件')
    init()

def init():
    global has_init
    if os.path.isfile(constants.CONFIG_PATH):
        logger.critical("错误：{} 应该是个目录，而不应该是个文件".format(constants.CONFIG_PATH))
    if not os.path.exists(constants.CONFIG_PATH):
        os.makedirs(constants.CONFIG_PATH)
    if not os.path.exists(constants.getConfigPath()):
        yes_no = input("配置文件{}不存在，要创建吗？(y/n)".format(constants.getConfigPath()))
        if yes_no.lower() == 'y':
            constants.newConfig()
            doInit(constants.getConfigPath())
        else:
            doInit(constants.getDefaultConfigPath())
    else:
        doInit(constants.getConfigPath())
    has_init = True

def doInit(config_file=constants.getDefaultConfigPath()):
    # Create config dir if it does not exist yet
    if not os.path.exists(constants.CONFIG_PATH):
        try:
            os.makedirs(constants.CONFIG_PATH)
        except OSError:
            logger.error("Could not create config dir: '%s'",
                          constants.CONFIG_PATH, exc_info=True)
            raise

    # Check if config dir is writable
    if not os.access(constants.CONFIG_PATH, os.W_OK):
        logger.critical("Config dir %s is not writable. Dingdang " +
                         "won't work correctly.",
                         constants.CONFIG_PATH)

    global _config

    # Read config
    logger.debug("Trying to read config file: '%s'", config_file)
    try:
        with open(config_file, "r") as f:
            _config = yaml.safe_load(f)
    except Exception as e:
        logger.error("配置文件 {} 读取失败: {}".format(config_file, e))
        raise


def get_path(items, default=None, warn=False):
    global _config
    curConfig = _config
    if isinstance(items, str) and items[0] == '/':
        items = items.split('/')[1:]
    for key in items:
        if key in curConfig:
            curConfig = curConfig[key]
        else:
            if warn:
                logger.warning("/%s not specified in profile, defaulting to "
                             "'%s'", '/'.join(items), default)
            else:
                logger.debug("/%s not specified in profile, defaulting to "
                             "'%s'", '/'.join(items), default)
            return default
    return curConfig


def has_path(items):
    global _config
    curConfig = _config
    if isinstance(items, str) and items[0] == '/':
        items = items.split('/')[1:]
    else:
        items = [items]
    for key in items:
        if key in curConfig:
            curConfig = curConfig[key]
        else:
            return False
    return True


def has(item):
    """
    判断配置里是否包含某个配置项

    :param item: 配置项名
    :returns: True: 包含; False: 不包含
    """
    return has_path(item)


def get(item='', default=None, warn=False):
    """
    获取某个配置的值

    :param item: 配置项名。如果是多级配置，则以 "/a/b" 的形式提供
    :param default: 默认值（可选）
    :param warn: 不存在该配置时，是否告警
    :returns: 这个配置的值。如果没有该配置，则提供一个默认值
    """
    global has_init
    if not has_init:
        init()
    if not item:
        return _config
    if item[0] == '/':
        return get_path(item, default, warn)
    try:
        return _config[item]
    except KeyError:
        if warn:
            logger.warning("%s not specified in profile, defaulting to '%s'",
                         item, default)
        else:
            logger.debug("%s not specified in profile, defaulting to '%s'",
                         item, default)
        return default
    
def getConfig():
    """
    返回全部配置数据

    :returns: 全部配置数据（字典类型）
    """
    return _config

def getText():
    if os.path.exists(constants.getConfigPath()):
        with open(constants.getConfigPath(), 'r') as f:
            return f.read()
    return ''

def dump(configStr):
    with open(constants.getConfigPath(), 'w') as f:
        f.write(configStr)




# -*- coding: utf-8-*-
import os
import shutil

# Wukong main directory
APP_PATH = os.path.normpath(os.path.join(
    os.path.dirname(os.path.abspath(__file__)), os.pardir))
 
LIB_PATH = os.path.join(APP_PATH, "robot")
DATA_PATH = os.path.join(APP_PATH, "static")
TEMP_PATH = os.path.join(APP_PATH, "temp")
TEMPLATE_PATH = os.path.join(APP_PATH, "server", "templates")
PLUGIN_PATH = os.path.join(APP_PATH, "plugins")
DEFAULT_CONFIG_NAME = 'default.yml'
CUSTOM_CONFIG_NAME = 'config.yml'

CONFIG_PATH = os.path.expanduser(
    os.getenv('WUKONG_CONFIG', '~/.wukong')
)
CONTRIB_PATH = os.path.expanduser(
    os.getenv('WUKONG_CONFIG', '~/.wukong/contrib')
)
CUSTOM_PATH = os.path.expanduser(
    os.getenv('WUKONG_CONFIG', '~/.wukong/custom')
)

def getConfigPath():
    """
    获取配置文件的路径

    returns: 配置文件的存储路径
    """
    return os.path.join(CONFIG_PATH, CUSTOM_CONFIG_NAME)

def getQAPath():
    """
    获取QA数据集文件的路径

    returns: QA数据集文件的存储路径
    """
    qa_source = os.path.join(DATA_PATH, 'qa.csv')
    qa_dst = os.path.join(CONFIG_PATH, 'qa.csv')
    if not os.path.exists(qa_dst):
        shutil.copyfile(qa_source, qa_dst)
    return qa_dst

def getConfigData(*fname):
    """
    获取配置目录下的指定文件的路径

    :param *fname: 指定文件名。如果传多个，则自动拼接
    :returns: 配置目录下的某个文件的存储路径
    """
    return os.path.join(CONFIG_PATH, *fname)

def getData(*fname):
    """
    获取资源目录下指定文件的路径

    :param *fname: 指定文件名。如果传多个，则自动拼接
    :returns: 配置文件的存储路径
    """
    return os.path.join(DATA_PATH, *fname)

def getDefaultConfigPath():    
    return getData(DEFAULT_CONFIG_NAME)

def newConfig():
    shutil.copyfile(getDefaultConfigPath(), getConfigPath())

def getHotwordModel(fname):
    if os.path.exists(getData(fname)):
        return getData(fname)
    else:
        return getConfigData(fname)




# -*- coding: utf-8-*-

import os
import tempfile
import wave
import shutil
import re
import time
import json
import yaml
import hashlib
import subprocess
from . import constants, config
from robot import logging
from pydub import AudioSegment
from pytz import timezone
import _thread as thread

import smtplib
from email.mime.text import MIMEText
from email.mime.multipart import MIMEMultipart

logger = logging.getLogger(__name__)

do_not_bother = False
is_recordable = True

def sendEmail(SUBJECT, BODY, ATTACH_LIST, TO, FROM, SENDER,
              PASSWORD, SMTP_SERVER, SMTP_PORT):
    """
    发送邮件

    :param SUBJECT: 邮件标题
    :param BODY: 邮件正文
    :param ATTACH_LIST: 附件
    :param TO: 收件人
    :param FROM: 发件人
    :param SENDER: 发件人信息
    :param PASSWORD: 密码
    :param SMTP_SERVER: smtp 服务器
    :param SMTP_PORT: smtp 端口号
    :returns: True: 发送成功; False: 发送失败
    """
    txt = MIMEText(BODY.encode('utf-8'), 'html', 'utf-8')
    msg = MIMEMultipart()
    msg.attach(txt)    

    for attach in ATTACH_LIST:
        try:
            att = MIMEText(open(attach, 'rb').read(), 'base64', 'utf-8')
            filename = os.path.basename(attach)
            att["Content-Type"] = 'application/octet-stream'
            att["Content-Disposition"] = 'attachment; filename="%s"' % filename
            msg.attach(att)
        except Exception:
            logger.error(u'附件 %s 发送失败！' % attach)
            continue

    msg['From'] = SENDER
    msg['To'] = TO
    msg['Subject'] = SUBJECT

    try:
        session = smtplib.SMTP(SMTP_SERVER)
        session.connect(SMTP_SERVER, SMTP_PORT)
        session.starttls()
        session.login(FROM, PASSWORD)
        session.sendmail(SENDER, TO, msg.as_string())
        session.close()
        return True
    except Exception as e:
        logger.error(e)
        return False


def emailUser(SUBJECT="", BODY="", ATTACH_LIST=[]):
    """
    给用户发送邮件

    :param SUBJECT: subject line of the email
    :param BODY: body text of the email
    :returns: True: 发送成功; False: 发送失败
    """
    # add footer
    if BODY:
        BODY = u"%s，<br><br>这是您要的内容：<br>%s<br>" % (config['first_name'], BODY)

    recipient = config.get('/email/address', '')
    robot_name = config.get('robot_name_cn', 'wukong-robot')
    recipient = robot_name + " <%s>" % recipient
    user = config.get('/email/address', '')
    password = config.get('/email/password', '')
    server = config.get('/email/smtp_server', '')
    port = config.get('/email/smtp_port', '')

    if not recipient or not user or not password or not server or not port:
        return False
    try:
        sendEmail(SUBJECT, BODY, ATTACH_LIST, user, user,
                  recipient, password, server, port)
        return True
    except Exception as e:
        logger.error(e)
        return False


def get_file_content(filePath):
    """
    读取文件内容并返回

    :param filePath: 文件路径
    :returns: 文件内容
    :raises IOError: 读取失败则抛出 IOError
    """
    with open(filePath, 'rb') as fp:
        return fp.read()

def check_and_delete(fp, wait=0):
    """ 
    检查并删除文件/文件夹

    :param fp: 文件路径
    """
    def run():
        if wait > 0:
            time.sleep(wait)
        if isinstance(fp, str) and os.path.exists(fp):
            if os.path.isfile(fp):
                os.remove(fp)
            else:
                shutil.rmtree(fp)
    
    thread.start_new_thread(run, ())


def write_temp_file(data, suffix, mode='w+b'):
    """ 
    写入临时文件

    :param data: 数据
    :param suffix: 后缀名
    :param mode: 写入模式，默认为 w+b
    :returns: 文件保存后的路径
    """
    with tempfile.NamedTemporaryFile(mode=mode, suffix=suffix, delete=False) as f:
        f.write(data)
        tmpfile = f.name
    return tmpfile

def get_pcm_from_wav(wav_path):
    """ 
    从 wav 文件中读取 pcm
    
    :param wav_path: wav 文件路径
    :returns: pcm 数据
    """
    wav = wave.open(wav_path, 'rb')
    return wav.readframes(wav.getnframes())

def convert_wav_to_mp3(wav_path):
    """ 
    将 wav 文件转成 mp3

    :param wav_path: wav 文件路径
    :returns: mp3 文件路径
    """
    if not os.path.exists(wav_path):
        logger.critical("文件错误 {}".format(wav_path))
        return None
    mp3_path = wav_path.replace('.wav', '.mp3')
    AudioSegment.from_wav(wav_path).export(mp3_path, format="mp3")
    return mp3_path

def convert_mp3_to_wav(mp3_path):
    """ 
    将 mp3 文件转成 wav

    :param mp3_path: mp3 文件路径
    :returns: wav 文件路径
    """
    target = mp3_path.replace(".mp3", ".wav")
    if not os.path.exists(mp3_path):
        logger.critical("文件错误 {}".format(mp3_path))
        return None
    AudioSegment.from_mp3(mp3_path).export(target, format="wav")
    return target        

def clean():
    """ 清理垃圾数据 """
    temp = constants.TEMP_PATH
    temp_files = os.listdir(temp)
    for f in temp_files:
        if os.path.isfile(os.path.join(temp, f)) and re.match(r'output[\d]*\.wav', os.path.basename(f)):
            os.remove(os.path.join(temp, f))

def setRecordable(value):
    """ 设置是否可以开始录制语音 """
    global is_recordable
    is_recordable = value

def isRecordable():
    """ 是否可以开始录制语音 """
    global is_recordable
    return is_recordable

def is_proper_time():
    """ 是否合适时间 """
    global do_not_bother
    if do_not_bother == True:
        return False
    if not config.has('do_not_bother'):
        return True
    bother_profile = config.get('do_not_bother')
    if not bother_profile['enable']:
        return True
    if 'since' not in bother_profile or 'till' not in bother_profile:
        return True
    since = bother_profile['since']
    till = bother_profile['till']
    current = time.localtime(time.time()).tm_hour
    if till > since:
        return current not in range(since, till)
    else:
        return not (current in range(since, 25) or
                    current in range(-1, till))

def get_do_not_bother_on_hotword():
    """ 打开勿扰模式唤醒词 """
    return config.get('/do_not_bother/on_hotword', '悟空别吵.pmdl')

def get_do_not_bother_off_hotword():
    """ 关闭勿扰模式唤醒词 """
    return config.get('/do_not_bother/off_hotword', '悟空醒醒.pmdl')

def getTimezone():
    """ 获取时区 """
    return timezone(config.get('timezone', 'HKT'))

def getCache(msg):
    """ 获取缓存的语音 """
    md5 = hashlib.md5(msg.encode('utf-8')).hexdigest()
    mp3_cache = os.path.join(constants.TEMP_PATH, md5 + '.mp3')
    wav_cache = os.path.join(constants.TEMP_PATH, md5 + '.wav')
    if os.path.exists(mp3_cache):
        return mp3_cache
    elif os.path.exists(wav_cache):
        return wav_cache
    return None

def saveCache(voice, msg):
    """ 获取缓存的语音 """
    foo, ext = os.path.splitext(voice)
    md5 = hashlib.md5(msg.encode('utf-8')).hexdigest()
    target = os.path.join(constants.TEMP_PATH, md5+ext)
    shutil.copyfile(voice, target)
    return target
    

def lruCache():
    """ 清理最近未使用的缓存 """
    def run(*args):
        if config.get('/lru_cache/enable', True):            
            days = config.get('/lru_cache/days', 7)
            subprocess.run('find . -name "*.mp3" -atime +%d -exec rm {} \;' % days, cwd=constants.TEMP_PATH, shell=True)

    thread.start_new_thread(run, ())

def validyaml(filename):
    """
    校验 YAML 格式是否正确

    :param filename: yaml文件路径
    :returns: True: 正确; False: 不正确
    """
    try:
        f = open(filename)
        str = f.read()
        yaml.safe_load(str)
        return True
    except Exception:
        return False

def validjson(s):
    """
    校验某个 JSON 字符串是否正确
    
    :param s: JOSN字符串
    :returns: True: 正确; False: 不正确
    """
    try:
        json.loads(s)
        return True
    except Exception:
        return False

def stripPunctuation(s):
    """
    移除字符串末尾的标点
    """
    punctuations = [',', '，', '.', '。', '?']
    if any(s.endswith(p) for p in punctuations):
        s = s[:-1]
    return s




# -*- coding:utf-8 -*-
from __future__ import print_function
import RPi.GPIO as GPIO
import threading
import time


class Motor(object):
    def __init__(self):
        self.INT3 = 16
        self.INT4 = 20
        self.ENB = 21

    def _move_forward_(self, duration, backforward=False):
        try:
            GPIO.setmode(GPIO.BCM)
            GPIO.setup(self.INT3, GPIO.OUT)
            GPIO.setup(self.INT4, GPIO.OUT)
            GPIO.setup(self.ENB, GPIO.OUT)
            pwmb = GPIO.PWM(self.ENB, 80)
            pwmb.start(50)
            if backforward:
                GPIO.output(self.INT3, GPIO.HIGH)
                GPIO.output(self.INT4, GPIO.LOW)
            else:
                GPIO.output(self.INT3, GPIO.LOW)
                GPIO.output(self.INT4, GPIO.HIGH)
            pwmb.ChangeDutyCycle(90)
            time.sleep(duration)
        finally:
            pwmb.stop()  # 停止PWM
            GPIO.cleanup()  # 清理释放GPIO资源，将GPIO复位

    def move_forward(self, duration, backforward=False, wait=True):
        if not wait:
            t1 = threading.Thread(target=self._move_forward_, args=(duration, backforward, ))
            t1.start()
        else:
            self._move_forward_(duration, backforward)


if __name__ == "__main__":
    motor = Motor()
    motor.move_forward(2, backforward=True, wait=True)



#!/usr/bin/env python
#-*- coding:utf-8 -*-

#############################################
#  File Name: setup.py
#  Author: xialingming
#  Mail: xialingming@gmail.com
#  #############################################


from setuptools import setup, find_packages

setup(
    name="kdxfsdk",
    version="0.0.33",
    keywords=("kdxf", "sdk", "xialingming"),
    description="kdxf sdk",
    long_description="xf sdk for python",
    license="MIT Licence",

    url="http://xialingming",
    author="lmxia",
    author_email="xialingming@gmail.com",
    packages=find_packages(),
    include_package_data=True,
    platforms="any",
    install_requires=["pyaudio", "pyyaml>=4.2b1", "requests==2.21.0", "websocket-client==0.56.0",
                      "python-dateutil==2.7.5", "jieba", "pypinyin", "websocket==0.2.1", "semver==2.8.1",
                      "markdown==3.0.1", "tornado==5.1.1", "fire==0.1.3", "pytz==2018.9", "watchdog==0.9.0",
                      "pydub==0.23.1", "baidu-aip==2.0.0.1"]
)